__Description__ = '''This Script parses the Stata results and calculates corresponding ad valorem equivalent values
                     for the TradeProd dataset.'''

# Import Packages
import pandas as pd
import os as os
import numpy as np

# ------
# Specification
# ------

# INPUTS:
# Directory containing sector results from step 2a
sector_results_directory = "goods_estimation_code_release\\tradeprod_sector_estimates"
# File containing aggregate results from step 2a
aggregate_local = "goods_estimation_code_release\\aggregate_estimates\\tradeprod_estimates_aggregate.dta"
# Database of sigma values
sigma_local = "goods_estimation_code_release/elasticity_values.txt"

# OUTPUTS:
# Save location
save_local = "goods_estimation_code_release/ave_estimates"

# Set dataset label for results
dataset = "tradeprod"

# ----
# Load Aggregate Estimates
# ----

agg_results = pd.read_stata(aggregate_local)
agg_results['gtap']='aggregate'

# ----
# Load and combine Sector estimates
# ----

# Collect file names from estimate output directory
result_files = os.listdir(sector_results_directory)
raw_datas = list()
# For each results file: load data, get sector name from the file name and add as a column, then store in a list.
for file in result_files:
    sector_results = pd.read_stata(os.path.join(sector_results_directory, file))
    sector = file[20:23]
    sector_results['gtap']=sector
    raw_datas.append(sector_results)
# Combine all the sector results + aggregate results
results = pd.concat(raw_datas+[agg_results], axis = 0)

# Drop rows corresponding to omitted variables/fixed effects
results = results.loc[~results['parm'].str.startswith('o.'),:]

# combined oil_gas sector gets labeled just 'oil' above, so create another corresponding 'gas' sector.
# Oil and gas diverge here due to differing elasticities
gas_results = results.loc[results['gtap']=='oil',:].copy()
gas_results['gtap'] = 'gas'
results = pd.concat([results, gas_results], axis = 0)



# ---
# Create ad valorem equivalent values
# ---
sigmas = pd.read_csv(sigma_local)

estimate_names = set(results['parm'].unique().tolist())
tpa_agreements = set(['agree_fta_no_tpa', 'CUSFTA', 'United_States_Israel', 'NAFTA', 'member_wto_joint',
                      'United_States_Jordan', 'United_States_Singapore', 'United_States_Chile',
                      'United_States_Australia',
                      'United_States_Morocco', 'United_States_Bahrain', 'CAFTA_DR', 'United_States_Oman',
                      'United_States_Peru', 'KORUS', 'United_States_Columbia',
                      'United_States_Panama']).intersection(estimate_names)


def get_aves(results, sigmas, variables=None, significance = 0):
    '''
    Calculate ad valorem equivalent values from coefficient estimates.
    '''
    var_col = 'parm'
    est_col = 'estimate'
    p_col = 'p'
    sigma_col = 'foreign_foreign'
    sect_col = 'gtap'
    ave_col = 'ave_reduction'
    if variables is None:
        variables = results[var_col].unique().tolist()

    # Lose insignificant results
    sig_results = results.copy()
    sig_results.loc[sig_results[p_col]>=(1-significance), est_col] = 0

    # add sigmas
    sigmas = sigmas[[sect_col,sigma_col]].copy()
    sig_results = sig_results.merge(sigmas, on = sect_col, how = 'left')

    # compute AVEs (Yotov et al. (2016) p. 30)
    sig_results['ave_reduction'] = 100*(np.exp(sig_results[est_col]/sig_results[sigma_col]) - 1)

    # reshape
    sig_results = sig_results.loc[sig_results[var_col].isin(variables), [var_col, ave_col, sect_col, p_col]]
    sig_results.set_index([sect_col, var_col], inplace = True)
    sig_results = sig_results.unstack()
    # Reorder columns based on agreement in second position of column tuple
    col_order = sorted(sig_results, key = lambda x: x[1])
    sig_results = sig_results[col_order]
    return sig_results


ave_results = get_aves(results, sigmas, variables= tpa_agreements)

ave_results.to_csv("{}\\ave_estimates_{}.csv".format(save_local,dataset))














